@@ -1,9 +1,9 @@
 # coding=utf-8
 #
 # This file is part of Hypothesis, which may be found at
-# https://github.com/HypothesisWorks/hypothesis-python
+# https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2018 David R. MacIver
+# Most of this work is copyright (C) 2013-2019 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -11,22 +11,29 @@
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
-# obtain one at http://mozilla.org/MPL/2.0/.
+# obtain one at https://mozilla.org/MPL/2.0/.
 #
 # END HEADER
 
-from __future__ import division, print_function, absolute_import
+from __future__ import absolute_import, division, print_function
 
 import enum
-import math
-import heapq
 import hashlib
+import heapq
+from collections import OrderedDict
 from fractions import Fraction
-from collections import Sequence, OrderedDict
 
 from hypothesis._settings import note_deprecation
-from hypothesis.internal.compat import floor, hbytes, hrange, qualname, \
-    bit_length, str_to_bytes, int_from_bytes
+from hypothesis.internal.compat import (
+    abc,
+    bit_length,
+    floor,
+    hbytes,
+    hrange,
+    int_from_bytes,
+    qualname,
+    str_to_bytes,
+)
 from hypothesis.internal.floats import int_to_float
 
 LABEL_MASK = 2 ** 64 - 1
@@ -49,17 +56,20 @@ def combine_labels(*labels):
     return label
 
 
-INTEGER_RANGE_DRAW_LABEL = calc_label_from_name(
-    'another draw in integer_range()')
-GEOMETRIC_LABEL = calc_label_from_name('geometric()')
-BIASED_COIN_LABEL = calc_label_from_name('biased_coin()')
-SAMPLE_IN_SAMPLER_LABLE = calc_label_from_name('a sample() in Sampler')
-ONE_FROM_MANY_LABEL = calc_label_from_name('one more from many()')
+INTEGER_RANGE_DRAW_LABEL = calc_label_from_name("another draw in integer_range()")
+BIASED_COIN_LABEL = calc_label_from_name("biased_coin()")
+SAMPLE_IN_SAMPLER_LABLE = calc_label_from_name("a sample() in Sampler")
+ONE_FROM_MANY_LABEL = calc_label_from_name("one more from many()")
 
 
 def integer_range(data, lower, upper, center=None):
     assert lower <= upper
     if lower == upper:
+        # Write a value even when this is trival so that when a bound depends
+        # on other values we don't suddenly disappear when the gap shrinks to
+        # zero - if that happens then often the data stream becomes misaligned
+        # and we fail to shrink in cases where we really should be able to.
+        data.write(hbytes([0]))
         return int(lower)
 
     if center is None:
@@ -97,42 +107,42 @@ def integer_range(data, lower, upper, center=None):
     return int(result)
 
 
-def centered_integer_range(data, lower, upper, center):
-    return integer_range(
-        data, lower, upper, center=center
-    )
+try:
+    from numpy import ndarray
+except ImportError:  # pragma: no cover
+    ndarray = ()
 
 
-def check_sample(values):
-    try:
-        from numpy import ndarray
-        if isinstance(values, ndarray):
-            if values.ndim != 1:
-                note_deprecation((
-                    'Only one-dimensional arrays are supported for sampling, '
-                    'and the given value has {ndim} dimensions (shape '
-                    '{shape}).  This array would give samples of array slices '
-                    'instead of elements!  Use np.ravel(values) to convert '
-                    'to a one-dimensional array, or tuple(values) if you '
-                    'want to sample slices.  Sampling a multi-dimensional '
-                    'array will be an error in a future version of Hypothesis.'
-                ).format(ndim=values.ndim, shape=values.shape))
-            return tuple(values)
-    except ImportError:  # pragma: no cover
-        pass
-
-    if not isinstance(values, (OrderedDict, Sequence, enum.EnumMeta)):
+def check_sample(values, strategy_name):
+    if isinstance(values, ndarray):
+        if values.ndim != 1:
+            note_deprecation(
+                (
+                    "Only one-dimensional arrays are supported for sampling, "
+                    "and the given value has {ndim} dimensions (shape "
+                    "{shape}).  This array would give samples of array slices "
+                    "instead of elements!  Use np.ravel(values) to convert "
+                    "to a one-dimensional array, or tuple(values) if you "
+                    "want to sample slices.  Sampling a multi-dimensional "
+                    "array will be an error in a future version of Hypothesis."
+                ).format(ndim=values.ndim, shape=values.shape),
+                since="2018-05-09",
+            )
+    elif not isinstance(values, (OrderedDict, abc.Sequence, enum.EnumMeta)):
         note_deprecation(
-            ('Cannot sample from %r, not a sequence.  ' % (values,)) +
-            'Hypothesis goes to some length to ensure that sampling an '
-            'element from a collection (with `sampled_from` or `choices`) is '
-            'replayable and can be minimised.  To replay a saved example, '
-            'the sampled values must have the same iteration order on every '
-            'run - ruling out sets, dicts, etc due to hash randomisation.  '
-            'Most cases can simply use `sorted(values)`, but mixed types or '
-            'special values such as math.nan require careful handling - and '
-            'note that when simplifying an example, Hypothesis treats '
-            'earlier values as simpler.')
+            "Cannot sample from {values}, not an ordered collection. "
+            "Hypothesis goes to some length to ensure that the {strategy} "
+            "strategy has stable results between runs. To replay a saved "
+            "example, the sampled values must have the same iteration order "
+            "on every run - ruling out sets, dicts, etc due to hash "
+            "randomisation. Most cases can simply use `sorted(values)`, but "
+            "mixed types or special values such as math.nan require careful "
+            "handling - and note that when simplifying an example, "
+            "Hypothesis treats earlier values as simpler.".format(
+                values=repr(values), strategy=strategy_name
+            ),
+            since="2017-04-12",
+        )
     return tuple(values)
 
 
@@ -152,21 +162,7 @@ FULL_FLOAT = int_to_float(FLOAT_PREFIX | ((2 << 53) - 1)) - 1
 
 
 def fractional_float(data):
-    return (
-        int_to_float(FLOAT_PREFIX | getrandbits(data, 52)) - 1
-    ) / FULL_FLOAT
-
-
-def geometric(data, p):
-    denom = math.log1p(-p)
-    data.start_example(GEOMETRIC_LABEL)
-    while True:
-        probe = fractional_float(data)
-        if probe < 1.0:
-            result = int(math.log1p(-probe) / denom)
-            assert result >= 0, (probe, p, result)
-            data.stop_example()
-            return result
+    return (int_to_float(FLOAT_PREFIX | getrandbits(data, 52)) - 1) / FULL_FLOAT
 
 
 def boolean(data):
@@ -316,7 +312,8 @@ class Sampler(object):
             self.table[lo][1] = hi
             self.table[lo][2] = one - scaled_probabilities[lo]
             scaled_probabilities[hi] = (
-                scaled_probabilities[hi] + scaled_probabilities[lo]) - one
+                scaled_probabilities[hi] + scaled_probabilities[lo]
+            ) - one
 
             if scaled_probabilities[hi] < 1:
                 heapq.heappush(small, hi)
@@ -382,6 +379,8 @@ class many(object):
         self.drawn = True
         self.rejected = False
 
+        self.data.start_example(ONE_FROM_MANY_LABEL)
+
         if self.min_size == self.max_size:
             should_continue = self.count < self.min_size
         elif self.force_stop:
@@ -396,10 +395,10 @@ class many(object):
             should_continue = biased_coin(self.data, p_continue)
 
         if should_continue:
-            self.data.start_example(ONE_FROM_MANY_LABEL)
             self.count += 1
             return True
         else:
+            self.data.stop_example()
             return False
 
     def reject(self):
